﻿//author: Alan Pipitone
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.IO;
using System.Management;
using System.Collections;
using System.Threading;


namespace check_defrag
{
    class Program
    {
        static void Main(string[] args)
        {

            ArrayList volumes = new ArrayList();
            ArrayList volumesCritical = new ArrayList();

            string tmpOutOk = "";
            string tmpOutCritical = "";
            string performance = "";
            string allVolumes = "";

            try
            {
                //read the arguments
                for (int i = 0; i < args.Length; i++)
                {
                    //check if the user has set a disk unit (example c: or d: etc.)
                    if (args[i].IndexOf(":") != -1)
                    {
                        volumes.Add(args[i]); //save the disk unit

                        allVolumes = allVolumes + args[i] + ","; //set the string for the output
                    }
                    else //else print the manual
                    {
                        Manual();
                        Environment.Exit(3);
                    }
                }

                if (args.Length == 0)
                {
                    Manual();
                    Environment.Exit(3);
                }

                //tranform allVolumes => EXAMPLE: transform "c:,d:,f:," to "c:,d: and f:"  
                if ((allVolumes.Split(',').Length - 1) > 1)
                {
                    allVolumes = allVolumes.Remove(allVolumes.LastIndexOf(","));
                    allVolumes = allVolumes.Insert(allVolumes.LastIndexOf(","), " and ");
                    allVolumes = allVolumes.Remove(allVolumes.LastIndexOf(","), 1);
                }
                else //tranform allVolumes => EXAMPLE: transform "c:," to "c:"  
                {
                    allVolumes = allVolumes.Remove(allVolumes.LastIndexOf(","));
                }

                //create objects for WMI 
                ManagementScope scope = new ManagementScope("\\\\" + Environment.MachineName + "\\root\\CIMV2");
                ObjectQuery query;

                foreach (string volume in volumes)
                {
                    //set WMI query
                    query = new ObjectQuery("SELECT * FROM Win32_Volume WHERE Name = '" + volume + "\\\\'");
                    scope.Connect();

                    ManagementObjectSearcher searcher = new ManagementObjectSearcher(scope, query);

                    object[] mngOutputs = new object[2];

                    foreach (ManagementObject diskUnit in searcher.Get())
                    {
                        //call Defrag Analysis
                        UInt32 result = (UInt32)diskUnit.InvokeMethod("DefragAnalysis", mngOutputs);

                        if (result == 0)
                        {
                            ManagementBaseObject mngAnalysis = (ManagementBaseObject)mngOutputs[1];

                            if (mngAnalysis != null)
                            {
                                //if we have to defrag the current disk unit
                                if (mngOutputs[0].ToString().ToLower() == "true")
                                {
                                    volumesCritical.Add(volume); //save current disk unit into volumesCritical
                                    tmpOutCritical = tmpOutCritical + volume + "(" + mngAnalysis["FilePercentFragmentation"].ToString() + "%),"; //set the string for the output
                                }
                                else
                                {
                                    tmpOutOk = tmpOutOk + volume + ","; //set the string for the output
                                }
                                
                                //set the string for the output with the performance data
                                performance = performance + volume.Replace(":", "") + "_file_fragmentation=" + mngAnalysis["FilePercentFragmentation"].ToString() + "% ";
                                    
                            }
                        }
                        else //the Defrag Analysis has returned an error
                        {
                            string errorCode = "";

                            switch (result)
                            {
                                case 1:
                                    errorCode = "Access Denied";
                                    break;
                                case 2:
                                    errorCode = "Defrag Analysis is Not Supported";
                                    break;
                                case 3:
                                    errorCode = "Volume Dirty Bit Is Set";
                                    break;
                                case 4:
                                    errorCode = "Not Enough Free Space";
                                    break;
                                case 5:
                                    errorCode = "Corrupt Master File Table Detected";
                                    break;
                                case 6:
                                    errorCode = "Call Canceled";
                                    break;
                                case 7:
                                    errorCode = "Call Cancellation Request Too Late";
                                    break;
                                case 8:
                                    errorCode = "Defrag Engine Is Already Running";
                                    break;
                                case 9:
                                    errorCode = "Unable To Connect To Defrag Engine";
                                    break;
                                case 10:
                                    errorCode = "Defrag Engine Error";
                                    break;
                                case 11:
                                    errorCode = "Unknown Error";
                                    break;
                                default:
                                    errorCode = "Unknown Error";
                                    break;
                            }

                            Console.WriteLine("Defrag Engine has failed on disk {0}, return code is: {1} => {2}", volume, result, errorCode);
                            Environment.Exit(3);
                        }
                    }

                    //sleep one second
                    Thread.Sleep(1000);
                }


                //tranform tmpOutCritical => EXAMPLE: transform "c:,d:,f:," to "c:,d: and f:"  
                if ((tmpOutCritical.Split(',').Length - 1) > 1)
                {
                    tmpOutCritical = tmpOutCritical.Remove(tmpOutCritical.LastIndexOf(","));
                    tmpOutCritical = tmpOutCritical.Insert(tmpOutCritical.LastIndexOf(","), " and ");
                    tmpOutCritical = tmpOutCritical.Remove(tmpOutCritical.LastIndexOf(","), 1);
                }
                else if (tmpOutCritical.LastIndexOf(",") != -1)//tranform tmpOutCritical => EXAMPLE: transform "c:," to "c:"  
                {
                    tmpOutCritical = tmpOutCritical.Remove(tmpOutCritical.LastIndexOf(","));
                }


                //tranform tmpOutOk => EXAMPLE: transform "c:,d:,f:," to "c:,d: and f:"  
                if ((tmpOutOk.Split(',').Length - 1) > 1)
                {
                    tmpOutOk = tmpOutOk.Remove(tmpOutOk.LastIndexOf(","));
                    tmpOutOk = tmpOutOk.Insert(tmpOutOk.LastIndexOf(","), " and ");
                    tmpOutOk = tmpOutOk.Remove(tmpOutOk.LastIndexOf(","), 1);
                }
                else if (tmpOutOk.LastIndexOf(",") != -1)//tranform tmpOutOk => EXAMPLE: transform "c:," to "c:"  
                {
                    tmpOutOk = tmpOutOk.Remove(tmpOutOk.LastIndexOf(","));
                }

                //we have to defrag the volume(s)
                if (volumesCritical.Count > 0)
                {
                    //print the message
                    if (volumes.Count > 1)
                    {
                        Console.WriteLine("you are checking " + allVolumes + ", you have to defrag " + tmpOutCritical + "|" + performance);
                    }
                    else
                    {
                        Console.WriteLine("you have to defrag " + tmpOutCritical + "|" + performance);
                    }
                    Environment.Exit(2);
                }
                else //we don't have to defrag the volume(s)
                {
                    //print the message
                    Console.WriteLine("you don't have to defrag " + tmpOutOk + "|" + performance);
                    Environment.Exit(0);
                }

                //if the execution is here then something has gone wrong
                Console.WriteLine("Unknown Error");
                Environment.Exit(3);

            }
            catch(Exception ex)
            {
                Console.WriteLine("An exception has occurred: {0}", ex.Message);
                Environment.Exit(3);
            }
        }


        private static void Manual()
        {
            string help = "Author: Alan Pipitone\n\n";
            help = help + "Website: http://www.alan-pipitone.com/\n\n";
            help = help + "Usage:\n\n";
            help = help + "check_defrag.exe <disk 1> [disk 2] [disk n]\n";
            help = help + "\n";
            help = help + "Examples:\n";
            help = help + "\n";
            help = help + "check_defrag.exe c:\n\n";
            help = help + "check_defrag.exe c: d:\n\n";
            Console.WriteLine(help);
        }
    }
}
